# Copyright 2015 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from cornice.resource import resource
from cornice.util import extract_json_data
import pyramid.httpexceptions as exc

from rutabaga import auth
from rutabaga.helpers import validate_uuid4


def validate_token_req(request):
    if request.method == 'POST':
        if 'username' not in request.json_body:
            request.errors.add('body', 'username', 'username is missing.')

    if request.method == 'DELETE':
        if 'secret' not in request.matchdict:
            request.errors.add(
                'querystring', 'secret', 'secret is missing.')
        if not validate_uuid4(request.matchdict['secret']):
            request.errors.add(
                'resource', 'secret', 'invalid secret.')
    return request


@resource(collection_path='/tokens', path='/tokens/{secret}',
          validators=validate_token_req)
class TokenAPI(object):

    def __init__(self, request):
        super(TokenAPI, self).__init__()
        self.request = request

    def collection_post(self):
        """Get a new time limited auth token.

        :param username: build id provided to buildd from builder.
        """
        username = extract_json_data(self.request).get('username')
        token = auth.create_token(username=username)
        auth.store_token(token)
        return token

    def delete(self):
        """Invalidate an existing token.
        :param secret: auth token id.
        """
        secret = self.request.matchdict['secret']
        token = auth.invalidate_token(secret)
        if not token:
            return exc.HTTPNotFound()
        return exc.HTTPOk()


def validate_validation_req(request):
    if 'username' not in request.json_body:
        request.errors.add('body', 'username', 'username is missing.')

    if 'secret' not in request.json_body:
        request.errors.add('body', 'secret', 'secret is missing.')
    return request


@resource(path='/validate', validators=validate_validation_req)
class TokenValidationAPI(object):

    def __init__(self, request):
        super(TokenValidationAPI, self).__init__()
        self.request = request

    def post(self):
        """Validate a token (username and secret pair).

        :param username: username for existing token.
        :param secret: secret of existing token.
        """
        username = extract_json_data(self.request).get('username')
        secret = extract_json_data(self.request).get('secret')
        token = auth.retrieve_token(username, secret)
        if not token:
            return exc.HTTPNotFound()
        return {'username': token['username']}
